/*
 * Created on 2-aug-2006
 */
package be.ASPL.presentationlayer;

import java.util.Locale;

import be.SIRAPRISE.util.MyMessageFormat;

/**
 * DynamicColumnList is the Presentation layer attribute type for lists whose columns cannot be defined in the .pres file, and cannot be but obtained from the data that the user function provides
 * 
 * @author Erwin Smout
 */
public class DynamicColumnList extends PresentationType<DynamicColumnListValue> {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#checkValidWaarValue(be.ASPL.presentationlayer.WaarValue)
	 */
	@Override
	public DynamicColumnListValue checkValidWaarValue (PresentationValue value, UserData userData) throws InvalidAttributeValueException {
		if (!(value instanceof ListValue || value instanceof DynamicColumnListValue)) {
			Locale locale = userData.getLocale();
			throw new InvalidAttributeValueException(value.toASPLString(userData), this.getClass().getSimpleName(), MyMessageFormat.format(I18N.getString(locale, "DynamicColumnList.InvalidValueClass"), new String[] { getClass().getName(), value.getClass().getName() })); //$NON-NLS-1$
		}
		return (DynamicColumnListValue) value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#encode(byte[], be.ASPL.presentationlayer.UserData, java.util.Locale)
	 */
	@Override
	public DynamicColumnListValue encode (byte[] partBytes, UserData userData, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.AttributeType#encode(java.lang.String)
	 */
	public DynamicColumnListValue encode (String val, UserData userData, Locale locale) throws InvalidAttributeValueException {
		throw new InvalidAttributeValueException(val, this.getClass().getSimpleName(), I18N.getString(locale, "DynamicColumnList.NoEncoding")); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getDefaultPresentationRenderingStyle()
	 */
	@Override
	public Renderer getDefaultPresentationRenderingStyle ( ) {
		return VariableColumnListRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationAttributeType#decode(be.erwinsmout.ASPL.WaarValue)
	 */
	public String getNakedStringRepresentation (PresentationValue presentationValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale) {
		return Integer.toString(((DynamicColumnListValue) presentationValue).getColumnList().size());
		//		String fieldHtml = ""; //$NON-NLS-1$
		// DynamicColumnListValue dynamicColumnListValue = (DynamicColumnListValue) presentationValue;
		//
		// LinkedList<String> columnList = dynamicColumnListValue.getColumnList();
		// int columnCount = Math.max(columnList.size(), 1);
		// int columnWidthPercent = 100 / columnCount;
		//		fieldHtml += "<tr>"; //$NON-NLS-1$
		// Iterator<String> i_columnList = columnList.iterator();
		// while (i_columnList.hasNext()) {
		// String columnName = i_columnList.next();
		//			fieldHtml += "<TH width=\"" + columnWidthPercent + "%\">" + columnName + "</th>"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		// }
		//		fieldHtml += "</tr>"; //$NON-NLS-1$
		//
		// LinkedList<HashMap<String, WaarValue>> rowList = dynamicColumnListValue.getRowList();
		// // iterate over the all the entries in the list
		// Iterator<HashMap<String, WaarValue>> i_rowList = rowList.iterator();
		// while (i_rowList.hasNext()) {
		// // get the valuemap that holds the row
		// HashMap<String, WaarValue> row = i_rowList.next();
		//			StringBuffer rowHtml = new StringBuffer("<tr>"); //$NON-NLS-1$
		// // reset the column iterator
		// i_columnList = columnList.iterator();
		// while (i_columnList.hasNext()) {
		// String columnName = i_columnList.next();
		// WaarValue entryColumnValue = row.get(columnName);
		// if (entryColumnValue != null) {
		// if (entryColumnValue instanceof GenericStringValue) {
		//						rowHtml.append("<td>" + ((GenericStringValue) entryColumnValue).getString() + "</td>"); //$NON-NLS-1$ //$NON-NLS-2$
		// } else {
		//						rowHtml.append("<td><table border=\"1\" width=\"100%\"><TBODY>" + getNakedStringRepresentation(entryColumnValue, userFunction, presentationField, locale) + "</TBODY></table></td>"); //$NON-NLS-1$ //$NON-NLS-2$
		// }
		// } else {
		//					rowHtml.append("<td> </td>"); //$NON-NLS-1$
		// }
		// }
		//			rowHtml.append("</tr>"); //$NON-NLS-1$
		// fieldHtml += rowHtml;
		// }
		//
		// return fieldHtml;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getPresentationRenderingStyle(java.lang.String)
	 */
	@Override
	public Renderer getPresentationRenderingStyle (String renderingStyleName) {
		return VariableColumnListRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(boolean, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (boolean b, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(int)
	 */
	@Override
	public PresentationValue getWaarValue (int v, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(long)
	 */
	@Override
	public PresentationValue getWaarValue (long v, Locale locale) {
		throw new UnsupportedOperationException();
	}
}