package be.ASPL.presentationlayer;

import java.io.*;

import be.SIRAPRISE.util.ConjunctFileNameFilterSet;
import be.SIRAPRISE.util.PrefixFilter;
import be.SIRAPRISE.util.SuffixFilter;

/**
 * @author Erwin
 * 
 */
public class PresFldToClassConverter {

	/**
	 * 
	 */
	private static final String PRES = ".pres"; //$NON-NLS-1$

	/**
	 * @param args
	 *            -
	 */
	public static void main (String[] args) {

		final String packageName = "be.SIRAPRISE.webclient"; //$NON-NLS-1$
		final String sourcePathComponentForPackage = packageName.replace('.', '/');
		final String sourceFolderName = "Sources/" + sourcePathComponentForPackage; //$NON-NLS-1$
		final String[] presentationDefFiles = new File(sourceFolderName).list(new SuffixFilter(PRES));

		for (int i1 = 0; i1 < presentationDefFiles.length; i1++) {
			final String presentationDefFileName = presentationDefFiles[i1];
			final String presentationName = presentationDefFileName.substring(0, presentationDefFileName.length() - PRES.length());
			final String newPresentationName = presentationName + "Presentation"; //$NON-NLS-1$
			final String presentationFileName = sourceFolderName + "/" + presentationDefFileName; //$NON-NLS-1$

			try (BufferedReader fin = new BufferedReader(new InputStreamReader(new FileInputStream(presentationFileName)))) {
				try (PrintStream newsource = new PrintStream(new FileOutputStream(sourceFolderName + "/" + newPresentationName + ".java"))) { //$NON-NLS-1$//$NON-NLS-2$

					newsource.println("package " + packageName + ";\r\nimport be.ASPL.presentationlayer.*;\r\nimport java.util.*;\r\n/**\r\n *\r\n */"); //$NON-NLS-1$ //$NON-NLS-2$
					newsource.println("public class " + newPresentationName + " extends Presentation {"); //$NON-NLS-1$ //$NON-NLS-2$
					newsource.println("\t/**\r\n\t * @param locale \r\n\t * @return\r\n\t * @throws ASPLException \r\n\t */"); //$NON-NLS-1$
					newsource.println("\tprivate static final java.util.List<PresentationField> getPresentationFields(Locale locale) throws ASPLException {"); //$NON-NLS-1$
					newsource.println("\t\treturn Arrays.asList(new PresentationField[]{ // "); //$NON-NLS-1$
					String sep = ""; //$NON-NLS-1$

					String l;
					String actionsSep = ""; //$NON-NLS-1$
					String actionsString = ""; //$NON-NLS-1$
					while ((l = fin.readLine()) != null) {
						if (!l.startsWith("# ")) { //$NON-NLS-1$
							if (l.length() >= "ACTION ".length() && l.substring(0, "ACTION ".length()).equalsIgnoreCase("ACTION ")) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
								actionsString = actionsString + actionsSep + "\"" + l.substring("ACTION ".length()).trim() + "\""; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
								actionsSep = ","; //$NON-NLS-1$
							} else {
								if (l.trim().length() > 0) {
									if (l.equalsIgnoreCase(")")) { //$NON-NLS-1$
										newsource.println("}))"); //$NON-NLS-1$
									} else {
										if (!l.equalsIgnoreCase("(")) { //$NON-NLS-1$
											try {
												newsource.println("\t\t" + sep + " new PresentationField(" + PresFldToClassConverter.getPresentationFieldConstructorArgumentsSourceText(l) + "null)  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ "); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
												sep = ","; //$NON-NLS-1$
											} catch (FieldIsListTypeException e) {
												newsource.println(sep + "\t\t new PresentationField(" + e.getArgsText() + " getPresentationFieldsMap(new PresentationField[]{  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ "); //$NON-NLS-1$//$NON-NLS-2$
												sep = ""; //$NON-NLS-1$
											}
										}
									}
								}
							}
						}
					}
					newsource.println("});\r\n\t}"); //$NON-NLS-1$

					newsource.print("\t/**\r\n\t *\r\n\t */\r\n\tprivate static final java.util.List<String> ACTIONS = Arrays.asList(new String[]{"); //$NON-NLS-1$
					newsource.println(actionsString + "});"); //$NON-NLS-1$
					newsource.println("\r\n\t/**\r\n\t * @param locale \r\n\t * @throws ASPLException\r\n\t */"); //$NON-NLS-1$
					newsource.println("\tpublic " + newPresentationName + " (Locale locale) throws ASPLException {"); //$NON-NLS-1$ //$NON-NLS-2$
					newsource.println("\t\tsuper(\"" + newPresentationName + "\",getPresentationFields(locale),ACTIONS,getPresentationLabels(\"" + newPresentationName + "\", locale, " + newPresentationName + ".class));  //$NON-NLS-1$ //$NON-NLS-2$"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					newsource.println("\t}\r\n}"); //$NON-NLS-1$
				}

				// Find all the labels properties files and rename those
				String[] labelsPropertiesFiles = new File(sourceFolderName).list(new ConjunctFileNameFilterSet(new PrefixFilter(presentationName), new SuffixFilter(".properties"))); //$NON-NLS-1$
				for (int i = 0; i < labelsPropertiesFiles.length; i++) {
					String newLabelsPropertiesFileName = newPresentationName + labelsPropertiesFiles[i].substring(presentationName.length());
					if (new File(sourceFolderName + File.separatorChar + labelsPropertiesFiles[i]).renameTo(new File(sourceFolderName + File.separatorChar + newLabelsPropertiesFileName))) {
						System.out.println(labelsPropertiesFiles[i] + " ---> " + newLabelsPropertiesFileName); //$NON-NLS-1$
					}
				}

				// Find the bodyhtml & formhtml files and rename those
				final String oldBodyHtmlName = presentationName + ".bodyhtml"; //$NON-NLS-1$
				final String newBodyHtmlName = newPresentationName + ".bodyhtml"; //$NON-NLS-1$
				if (new File(sourceFolderName + File.separatorChar + oldBodyHtmlName).renameTo(new File(sourceFolderName + File.separatorChar + newBodyHtmlName))) {
					System.out.println(oldBodyHtmlName + " ---> " + newBodyHtmlName); //$NON-NLS-1$
				}
				final String oldFormHtmlName = presentationName + ".formhtml"; //$NON-NLS-1$
				final String newformHtmlName = newPresentationName + ".formhtml"; //$NON-NLS-1$
				if (new File(sourceFolderName + File.separatorChar + oldFormHtmlName).renameTo(new File(sourceFolderName + File.separatorChar + newformHtmlName))) {
					System.out.println(oldFormHtmlName + " ---> " + newformHtmlName); //$NON-NLS-1$
				}
				final String[] tableRowHtmlFiles = new File(sourceFolderName).list(new ConjunctFileNameFilterSet(new PrefixFilter(presentationName), new SuffixFilter(".tablerowhtml"))); //$NON-NLS-1$
				for (int i = 0; i < tableRowHtmlFiles.length; i++) {
					final String newTablerowHtmlName = new StringBuilder(tableRowHtmlFiles[i]).insert(presentationName.length(), "Presentation").toString(); //$NON-NLS-1$
					if (new File(sourceFolderName + File.separatorChar + tableRowHtmlFiles[i]).renameTo(new File(sourceFolderName + File.separatorChar + newTablerowHtmlName))) {
						System.out.println(tableRowHtmlFiles[i] + " ---> " + newTablerowHtmlName); //$NON-NLS-1$
					}
				}

			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}

			// delete the .pres file
			if (new File(presentationFileName).delete()) {
				System.out.println(presentationFileName + " deleted."); //$NON-NLS-1$
			}
		}

	}

	/**
	 * @param presfldLine
	 * @return
	 * @throws FieldIsListTypeException
	 */
	private static String getPresentationFieldConstructorArgumentsSourceText (String presfldLine) throws FieldIsListTypeException {
		String[] words = presfldLine.split(",", 9); //$NON-NLS-1$

		if (words.length > 1 && words[1].equals(List.class.getName())) {
			throw new FieldIsListTypeException(getArgsText(words));
		}
		return getArgsText(words);
	}

	/**
	 * @param words
	 * @return
	 */
	private static String getArgsText (String[] words) {
		return ("\"" + words[0] + "\"," + words[1] + ".class, locale, \"" + (words.length > 2 ? words[2] : "") + "\", " //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$//$NON-NLS-5$
		+ (words.length > 3 && (words[3].equalsIgnoreCase("optional") || words[3].equalsIgnoreCase("0")) ? "false" : "true")) + ", " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				+ (words.length > 4 && words[4].trim().length() > 0 ? words[4] : "-1") + ", " //$NON-NLS-1$//$NON-NLS-2$
				+ (words.length > 5 && (words[5].trim().equalsIgnoreCase("input") || words[5].trim().equalsIgnoreCase("1")) ? "PresentationFieldIOMode.IN" : "PresentationFieldIOMode.OUT") + ", " //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
				+ "\"" + (words.length > 6 && words[6] != null && words[6].trim().length() > 0 ? words[6].trim() : "") + "\", " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				+ "\"" + (words.length > 7 ? words[7].trim() : "") + "\", " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				+ "\"" + (words.length > 8 ? words[8].trim() : "") + "\", "; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
	}
}
