package be.SIRAPRISE.birdwatcher;

import java.util.LinkedList;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.client.jsba.DmlAssignmentCommand;
import be.SIRAPRISE.client.jsba.DmlCommand;
import be.SIRAPRISE.client.jsba.DmlMultipleAssignmentCommand;
import be.SIRAPRISE.messages.ErrorMessageException;

/**
 * The function to create a new existing species
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = ExistingSpeciesCreatePresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = true, descriptionLocalizationKey = "ExistingSpeciesCreate.Text")
public class ExistingSpeciesCreate extends BWSpeciesFunction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		String speciesName = ((StringValue) getPresentationValue(ATTRIBUTENAMES.SPECIESNAME)).getString();
		StringValue language_wv = (StringValue) getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
		StringValue nlSpeciesName_wv = (StringValue) getPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME);

		LinkedList<DmlAssignmentCommand> commands = new LinkedList<DmlAssignmentCommand>();
		ExistingSpecies existingSpecies = new ExistingSpecies(speciesName);
		commands.add(existingSpecies.dbAddCommand(RELVARNAMES.EXISTINGSPECIES));

		if (((LogicalValue) getPresentationValue("MARKASPOSSIBLE")).getValue()) { //$NON-NLS-1$
			commands.add(existingSpecies.dbAddCommand(RELVARNAMES.POSSIBLESPECIES));
		}

		if (language_wv != null) {
			if (nlSpeciesName_wv != null) {
				commands.addLast(new NLSpeciesName(speciesName, language_wv.getString(), nlSpeciesName_wv.getString()).dbAddCommand(RELVARNAMES.NLSPECIESNAME));
			} else {
				throw new PleaseSpecifyBothLanguageAndNameException();
			}
		} else {
			if (nlSpeciesName_wv != null) {
				throw new PleaseSpecifyBothLanguageAndNameException();
			}
		}

		
		try (DBConnection dbc = getDBConnection();) {
			DmlCommand.execDmlCommandAndEndTransaction(dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.UPDATE), new DmlMultipleAssignmentCommand(commands)); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doResumeLogic()
	 */
	public void doResumeLogic ( ) throws ASPLException {
		doPresentationLogic();
	}
}