package be.SIRAPRISE.birdwatcher;

import java.util.LinkedList;
import java.util.Locale;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.jsba.ClassDoesNotImplementDBObjectException;
import be.SIRAPRISE.client.jsba.ConstructorMissingException;
import be.SIRAPRISE.client.jsba.DBObjectFactory;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.NameIdentifier;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.typeimplementations.Tuple;
import be.SIRAPRISE.util.BracketParser;

/**
 * The function to manage an existing species
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = ExistingSpeciesOverviewPresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = false, descriptionLocalizationKey = "ExistingSpeciesManage.Text")
public class ExistingSpeciesManage extends BWSpeciesFunction {

	/**
	 * 
	 */
	private static final String ISALSOPOSSIBLE = "ISALSOPOSSIBLE"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final NameIdentifier ISPOSSIBLE = NameIdentifier.get("ISPOSSIBLE"); //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String SPECIESLIST = "SPECIESLIST"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String NLSPECIESNAMELIST = "NLSPECIESNAMELIST"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final NameIdentifier NLSPECIESNAMELIST_I = NameIdentifier.get(NLSPECIESNAMELIST);

	/**
	 * @throws ASPLException
	 * 
	 */
	private void processOK ( ) throws ASPLException {

		
		try (DBConnection dbc = getDBConnection();) {
			ExistingSpecies species = (ExistingSpecies) getFromFunctionContextMandatory(RELVARNAMES.EXISTINGSPECIES);
			ExistingSpecies oldSpecies = (ExistingSpecies) species.getPreUpdateState();
			String newSpeciesName = ((StringValue) getPresentationValue(ATTRIBUTENAMES.SPECIESNAME)).getString();
			LinkedList<ServerSingleAssignmentCommand> commands = new LinkedList<ServerSingleAssignmentCommand>();
			PresentationValue language_wv = getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
			PresentationValue nlName_wv = getPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME);
			if (!newSpeciesName.equals(species.getSpeciesName())) {
				// species rename requested, do the rename, ignore other fields

				if (language_wv != null || nlName_wv != null) {
					throw new CannotEnterNLNamesWhileRenamingException();
				}

				species.setSpeciesName(newSpeciesName);
				try {
					commands.addLast(species.dbUpdateCommand(RELVARNAMES.EXISTINGSPECIES).getServerCommand());
				} catch (NoUpdatesException e) {
					// impossible due to the previous setName
					throw new RuntimeException(e);
				}

				commands.addLast(new ServerUpdateCommand(RELVARNAMES.NLSPECIESNAME, "RESTRICT(" + RELVARNAMES.NLSPECIESNAME + ",EQ(" + ATTRIBUTENAMES.SPECIESNAME + ",STRING(" + BracketParser.meta(oldSpecies.getSpeciesName()) + ")))", ATTRIBUTENAMES.SPECIESNAME + "(STRING(" + BracketParser.meta(newSpeciesName) + "))")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$

				if (!((LogicalValue) getPresentationValue(ISALSOPOSSIBLE)).getValue()) {
					commands.addLast(oldSpecies.dbUnAssertCommand(RELVARNAMES.POSSIBLESPECIES).getServerCommand());
				}
				commands.addLast(new ServerUpdateCommand(RELVARNAMES.POSSIBLESPECIES, "RESTRICT(" + RELVARNAMES.POSSIBLESPECIES + ",EQ(" + ATTRIBUTENAMES.SPECIESNAME + ",STRING(" + BracketParser.meta(oldSpecies.getSpeciesName()) + ")))", ATTRIBUTENAMES.SPECIESNAME + "(STRING(" + BracketParser.meta(newSpeciesName) + "))")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$

				if (((LogicalValue) getPresentationValue(ISALSOPOSSIBLE)).getValue()) {
					commands.addLast(species.dbAssertCommand(RELVARNAMES.POSSIBLESPECIES).getServerCommand());
				}
			} else {
				if (((LogicalValue) getPresentationValue(ISALSOPOSSIBLE)).getValue()) {
					commands.addLast(species.dbAssertCommand(RELVARNAMES.POSSIBLESPECIES).getServerCommand());
				} else {
					commands.addLast(species.dbUnAssertCommand(RELVARNAMES.POSSIBLESPECIES).getServerCommand());
				}
				if (language_wv != null) {
					if (nlName_wv != null) {
						commands.addLast(new ServerUpdateCommand(RELVARNAMES.NLSPECIESNAME, "RESTRICT(" + RELVARNAMES.NLSPECIESNAME + ",AND(EQ(" + ATTRIBUTENAMES.SPECIESNAME + ",STRING(" + BracketParser.meta(species.getSpeciesName()) + ")),EQ(LANGUAGE,LANGUAGE(ISO2(" + ((StringValue) language_wv).getString() + ")))))", ATTRIBUTENAMES.NLSPECIESNAME + "(" + BracketParser.meta(((StringValue) nlName_wv).getString()) + ")")); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
						commands.addLast(new NLSpeciesName(species.getSpeciesName(), ((StringValue) language_wv).getString(), ((StringValue) nlName_wv).getString()).dbAssertCommand(RELVARNAMES.NLSPECIESNAME).getServerCommand());
					} else {
						// delete the currently existing name for that language
						commands.addLast(new ServerDeleteCommand(RELVARNAMES.NLSPECIESNAME, "RESTRICT(" + RELVARNAMES.NLSPECIESNAME + ",AND(EQ(" + ATTRIBUTENAMES.SPECIESNAME + ",STRING(" + BracketParser.meta(species.getSpeciesName()) + ")),EQ(LANGUAGE,LANGUAGE(ISO2(" + ((StringValue) language_wv).getString() + ")))))")); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$ //$NON-NLS-5$
					}
				} else {
					if (nlName_wv != null) {
						throw new PleaseSpecifyBothLanguageAndNameException();
					}
				}
			}
			dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.UPDATE).execServerCommandAndEndTransaction(new ServerMultipleAssignmentCommand(commands)); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}

		refreshInfo(getMatchingSpecies());
		setPresentationValue(ATTRIBUTENAMES.LANGUAGE, ""); //$NON-NLS-1$
		setPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME, ""); //$NON-NLS-1$
	}

	/**
	 * @throws ASPLException
	 * 
	 */
	private void processSelectionPresentation ( ) throws ASPLException {

		Relation r = getMatchingSpecies();

		if (r.size() == 1) {
			setPresentation(new ExistingSpeciesManageDetailsPresentation());
			refreshInfo(r);
		} else {
			Locale locale = getUserLocale();
			setPresentationValue(SPECIESLIST, ASPLListFactory.getASPLListValue(getPresentation().getPresentationField(SPECIESLIST), r, getUserData(), locale));
		}

	}

	/**
	 * @param r
	 * @throws ASPLApplicationException
	 * @throws ASPLException
	 */
	private void refreshInfo (Relation r) throws ASPLApplicationException, ASPLException {
		ExistingSpecies species;
		try {
			species = DBObjectFactory.getObjectCollectionIgnoringMissingSetters(r, ExistingSpecies.class).iterator().next();
		} catch (ConstructorMissingException e) {
			throw new ASPLApplicationException(e);
		} catch (ClassDoesNotImplementDBObjectException e) {
			throw new ASPLApplicationException(e);
		}
		setPresentationValue(ATTRIBUTENAMES.SPECIESNAME, species.getSpeciesName());
		Tuple t = r.iterator().next();
		setPresentationValue(ISALSOPOSSIBLE, t.valueText(ISPOSSIBLE));

		Relation nlNames = t.rvaValue(NLSPECIESNAMELIST_I);
		Locale locale = getUserLocale();
		setPresentationValue(NLSPECIESNAMELIST, ASPLListFactory.getASPLListValue(getPresentation().getPresentationField(NLSPECIESNAMELIST), nlNames, getUserData(), locale));

		saveInFunctionContext(RELVARNAMES.EXISTINGSPECIES, species);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		setDefaultLanguageFromUserContext();
		if (getPresentationValue(ATTRIBUTENAMES.SPECIESNAME) != null || getPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME) != null || getPresentationValue("SPECIESNAMEPATTERN") != null || getPresentationValue("NLSPECIESNAMEPATTERN") != null) {//$NON-NLS-1$//$NON-NLS-2$
			processSelectionPresentation();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		if (getPresentation().getName().equalsIgnoreCase("ExistingSpeciesOverview")) { //$NON-NLS-1$
			processSelectionPresentation();
		} else {
			processOK();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doResumeLogic()
	 */
	public void doResumeLogic ( ) throws ASPLException {
		doPresentationLogic();
	}
}