package be.SIRAPRISE.birdwatcher;

import java.util.Date;
import java.util.LinkedList;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.client.jsba.DmlAssignmentCommand;
import be.SIRAPRISE.client.jsba.DmlCommand;
import be.SIRAPRISE.client.jsba.DmlMultipleAssignmentCommand;
import be.SIRAPRISE.messages.ErrorMessageException;

/**
 * The function to create a new existing species
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = SpotsCreatePresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = true, descriptionLocalizationKey = "SpotsCreate.Text")
public class SpotsCreate extends BWSpotsFunction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		Object walkDate = getFromSessionContext(ATTRIBUTENAMES.WALKDATE);
		if (walkDate instanceof DateValue) {
			setPresentationValue(ATTRIBUTENAMES.WALKDATE, (DateValue) walkDate);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		String speciesName = ((StringValue) getPresentationValue(ATTRIBUTENAMES.SPECIESNAME)).getString();
		Date walkDate = ((DateValue) getPresentationValue(ATTRIBUTENAMES.WALKDATE)).getDate();
		int spotCount = ((IntegerValue) getPresentationValue(ATTRIBUTENAMES.SPOTCOUNT)).getInteger();

		LinkedList<DmlAssignmentCommand> commands = new LinkedList<DmlAssignmentCommand>();

		if (((LogicalValue) getPresentationValue("FWALKDATE")).getValue()) { //$NON-NLS-1$
			commands.add(new Walk(walkDate).dbAssertCommand(RELVARNAMES.WALK));
		}

		if (((LogicalValue) getPresentationValue("FSPECIESNAME")).getValue()) { //$NON-NLS-1$
			commands.add(new ExistingSpecies(speciesName).dbAssertCommand(RELVARNAMES.POSSIBLESPECIES));
		}

		String gender = ((StringValue) getPresentationValue(ATTRIBUTENAMES.GENDER)).getString();
		if (gender.equalsIgnoreCase("-") || gender.equalsIgnoreCase(" ")) { //$NON-NLS-1$ //$NON-NLS-2$
			commands.add(new Spot(walkDate, speciesName, spotCount, ' ').dbAddCommand(RELVARNAMES.SPOTS));
		} else {
			commands.add(new Spot(walkDate, speciesName, spotCount, gender.charAt(0)).dbAddCommand(RELVARNAMES.GSPOTS));
		}
		
		try (DBConnection dbc = getDBConnection();) {
			DmlCommand.execDmlCommandAndEndTransaction(dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.UPDATE), new DmlMultipleAssignmentCommand(commands)); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}
}