/**
 * 
 */
package be.SIRAPRISE.tests.officialtestset;

import java.io.*;
import java.nio.ByteBuffer;
import java.util.*;
import java.util.Map.Entry;

import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.xml.sax.*;

import be.SIRAPRISE.client.*;
import be.SIRAPRISE.messages.*;
import be.SIRAPRISE.typeimplementations.*;
import be.SIRAPRISE.util.*;

/**
 * @author Erwin
 */
public class TestScenario implements Runnable {

	/**
	 * 
	 */
	private static final String ATTRIBUTENAME_AUTOCOMMIT = "autocommit"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String ATTRIBUTENAME_COMMIT = "commit"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String ATTRIBUTENAME_LABEL = "label"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String ATTRIBUTENAME_NAME = "name"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String ATTRIBUTENAME_READONLY = "readonly"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TAGNAME_ASSIGNMENT = "ASSIGNMENT"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TAGNAME_COMMIT = "COMMIT"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TAGNAME_INQUIRY = "INQUIRY"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TAGNAME_NESTEDTRANSACTION = "NESTEDTRANSACTION"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TAGNAME_OPERATION = "OPERATION"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TAGNAME_TRANSACTION = "TRANSACTION"; //$NON-NLS-1$

	/**
	 * 
	 */
	public static final int clientSideExceptionTypeID = 0x00c11e71;

	/**
	 * Optionname for naming the XML file containing the TestScenario to be executed
	 */
	public static final String FILE = "FILE"; //$NON-NLS-1$

	/**
	 * The HALTONFIRSTMISMATCH option is a boolean indicating whether or not execution of the test scenario is to halt immediately upon detection of the first mismatch between expected and actually obtained results
	 */
	public static final String HALTONFIRSTMISMATCH = "HALTONFIRSTMISMATCH"; //$NON-NLS-1$

	/**
	 * The default value for the {@link #HALTONFIRSTMISMATCH} option is {@value}
	 */
	public static final String HALTONFIRSTMISMATCHDEFAULT = "TRUE"; //$NON-NLS-1$

	/**
	 * @param div_float_float
	 * @param v1
	 * @param v2
	 * @return true if the two float values are close enough to be considered pragmatically equal
	 */
	private static boolean floatIntervalCloseEnough (final DIV_FLOAT_FLOAT div_float_float, ValueBuffer v1, ValueBuffer v2) {
		final ByteBuffer v1bb = v1.getByteBuffer();
		final ByteBuffer v2bb = v2.getByteBuffer();
		final short v1boundsindicator = v1bb.getShort();
		final short v2boundsindicator = v2bb.getShort();
		boolean v1lowerbounded = IntervalTypeImplementation.isLowerBounded(v1boundsindicator);
		boolean v2lowerbounded = IntervalTypeImplementation.isLowerBounded(v2boundsindicator);
		boolean v1upperbounded = IntervalTypeImplementation.isUpperBounded(v1boundsindicator);
		boolean v2upperbounded = IntervalTypeImplementation.isUpperBounded(v2boundsindicator);
		double v1b = 0;
		if (v1lowerbounded) {
			v1b = v1bb.getDouble();
		}
		double v2b = 0;
		if (v2lowerbounded) {
			v2b = v2bb.getDouble();
		}
		double v1e = 0;
		if (v1upperbounded) {
			v1e = v1bb.getDouble();
		}
		double v2e = 0;
		if (v2upperbounded) {
			v2e = v2bb.getDouble();
		}
		if (v1lowerbounded != v2lowerbounded) {
			return false;
		} else {
			if (v1lowerbounded) {
				if (!floatsCloseEnough(div_float_float, DbmsFloatImplementation.getValueBuffer(v1b), DbmsFloatImplementation.getValueBuffer(v2b))) {
					return false;
				}
			}
		}
		if (v1upperbounded != v2upperbounded) {
			return false;
		} else {
			if (v1upperbounded) {
				if (!floatsCloseEnough(div_float_float, DbmsFloatImplementation.getValueBuffer(v1e), DbmsFloatImplementation.getValueBuffer(v2e))) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * @param div_float_float
	 * @param v1
	 * @param v2
	 * @return
	 */
	private static boolean floatsCloseEnough (final DIV_FLOAT_FLOAT div_float_float, ValueBuffer v1, ValueBuffer v2) {
		return Math.abs(1.0 - DbmsFloatImplementation.getJavaDouble(div_float_float.executeOperator(MyLinkedList.createList(v1, v2)))) < 1E-9;
	}

	/**
	 * @param label
	 * @param commandText
	 * @return
	 */
	private static String getOperationIdentifier (String label, String commandText) {
		return (label == null || label.length() < 1) ? commandText : (label + " - " + commandText); //$NON-NLS-1$
	}

	/**
	 * @param results
	 * @param resultMisMatchDetected
	 * @param tx
	 * @param operationDetailElement
	 * @param label
	 * @return
	 * @throws ConnectionClosedException
	 */
	private static boolean processAssignmentTag (Collection<MyReadOnlyEntry<String, TestResultDetails>> results, DBTransaction tx, final Node operationDetailElement, String label) throws ConnectionClosedException {
		boolean resultMisMatchDetected = false;
		final NodeList assignmentDetailsElements = ((Element) operationDetailElement).getElementsByTagName("*"); //$NON-NLS-1$
		final String commandText = assignmentDetailsElements.item(0).getTextContent().trim(); // item(0) is the Command element
		// see if item(1) (Exception element) is present
		long expectedExceptionType = Long.MIN_VALUE;
		Collection<Long> expectedErrorNumbers = new LinkedList<>();
		String expectedExceptionClassName = ""; //$NON-NLS-1$
		String expectedExceptionMessage = ""; //$NON-NLS-1$
		boolean expectException = false;
		if (assignmentDetailsElements.getLength() > 1) {
			expectException = true;
			final Element expectedExceptionElement = (Element) assignmentDetailsElements.item(1); // ExpectedResult
			NodeList expectedExceptionDetailItems = expectedExceptionElement.getElementsByTagName("*"); //$NON-NLS-1$
			expectedExceptionType = Long.parseLong(expectedExceptionDetailItems.item(0).getTextContent().trim()); // ExceptionType tag
			int exceptionClassNameItemIndex;
			if (expectedExceptionDetailItems.getLength() > 1 && expectedExceptionDetailItems.item(1).getNodeName().endsWith(":ErrorNum")) { //$NON-NLS-1$
				final String allowableExpectedExceptionsCSV = expectedExceptionDetailItems.item(1).getTextContent().trim();
				expectedErrorNumbers = new CollectionTransformer<String, Long>(new StringToLongTransformer()).transformToSet(MyString.getNamesInEntryOrderWithoutDuplicates(allowableExpectedExceptionsCSV, ','));
				exceptionClassNameItemIndex = 2;
			} else {
				exceptionClassNameItemIndex = 1;
			}
			int exceptionMessageItemIndex;
			if (expectedExceptionDetailItems.getLength() > exceptionClassNameItemIndex && expectedExceptionDetailItems.item(exceptionClassNameItemIndex).getNodeName().endsWith(":ExceptionClass")) { //$NON-NLS-1$
				expectedExceptionClassName = expectedExceptionDetailItems.item(exceptionClassNameItemIndex).getTextContent();
				exceptionMessageItemIndex = exceptionClassNameItemIndex + 1;
			} else {
				expectedExceptionClassName = null;
				exceptionMessageItemIndex = exceptionClassNameItemIndex;
			}
			if (expectedExceptionDetailItems.getLength() > exceptionMessageItemIndex && expectedExceptionDetailItems.item(exceptionMessageItemIndex).getNodeName().endsWith(":ExceptionMessage")) { //$NON-NLS-1$
				expectedExceptionMessage = expectedExceptionDetailItems.item(exceptionMessageItemIndex).getTextContent();
			} else {
				expectedExceptionMessage = null;
			}
		}

		System.out.print('.');
		try {
			tx.execServerCommand(ServerCommand.getCommandObjectFrom(commandText));
			if (expectException) {
				resultMisMatchDetected = true;
				results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new TextOnlyTestResultDetails(MyMessageFormat.format("Execution successful, expected error {0}:{1}:{2}.", new String[] { Long.toString(expectedExceptionType), expectedErrorNumbers.toString(), expectedExceptionMessage })))); //$NON-NLS-1$
			}
		} catch (NotFoundException | IllegalServerCommandException e) {
			final String obtainedExceptionClassName = e.getClass().getName();
			// check the returned exception against the expected exception
			if (expectException) {
				final int obtainedExceptionType = clientSideExceptionTypeID;
				if (obtainedExceptionType != expectedExceptionType) {
					resultMisMatchDetected = true;
					results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionTypeMismatch(expectedExceptionType, obtainedExceptionType)));
				} else {
					if (!(e.getClass().getName().equals(expectedExceptionClassName))) {
						resultMisMatchDetected = true;
						results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionClassMismatch(expectedExceptionClassName, obtainedExceptionClassName)));
					} else {

					}
				}
			} else {
				resultMisMatchDetected = true;
				results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new UnexpectedExceptionDetails(e)));
			}
		} catch (ErrorMessageException e) {
			// check the returned exception against the expected exception
			final ServerErrorMessage serverErrorMessage = e.getServerErrorMessage();
			if (expectException) {
				final int obtainedExceptionType = serverErrorMessage.getMessageTypeID();
				if (obtainedExceptionType != expectedExceptionType) {
					resultMisMatchDetected = true;
					results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionTypeMismatch(expectedExceptionType, obtainedExceptionType, serverErrorMessage.getErrorClass())));
				} else {
					if (expectedExceptionType != NonSiraPriseErrorMessageType.MESSAGETYPEID && !expectedErrorNumbers.contains(new Long(serverErrorMessage.getErrorNumber()))) {
						resultMisMatchDetected = true;
						results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionNumberMismatch(expectedErrorNumbers, serverErrorMessage.getErrorNumber(), expectedExceptionMessage, serverErrorMessage.getClientResponseText())));
					} else {
						if (expectedExceptionType == NonSiraPriseErrorMessageType.MESSAGETYPEID && !(((NonSiraPriseErrorMessage) serverErrorMessage).getJavaThrowableClassName().equals(expectedExceptionClassName))) {
							resultMisMatchDetected = true;
							results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionClassMismatch(expectedExceptionClassName, ((NonSiraPriseErrorMessage) serverErrorMessage).getJavaThrowableClassName())));
						} else {

						}
					}
				}
			} else {
				resultMisMatchDetected = true;
				results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new UnexpectedExceptionDetails(e)));
			}
		}
		return resultMisMatchDetected;
	}

	/**
	 * @param tx
	 * @param operationDetailElement
	 * @throws ConnectionClosedException
	 */
	private static void processCommitTag (DBTransaction tx, final Node operationDetailElement) throws ConnectionClosedException {
		boolean commit = ((Element) operationDetailElement).hasAttribute(ATTRIBUTENAME_COMMIT) ? Boolean.parseBoolean(((Element) operationDetailElement).getAttribute(ATTRIBUTENAME_COMMIT)) : true;
		// System.out.println(commit);
		if (commit) {
			tx.commit();
		} else {
			tx.rollback();
		}
	}

	/**
	 * @param relation1
	 * @param relation2
	 * @param floatingPointDependentAttributes
	 * @return
	 */
	private static boolean resultsWithinExpectedMarginForFloats (Relation relation1, Relation relation2, Map<NameIdentifier, TypeDeclaration> floatingPointDependentAttributes) {
		final DIV_FLOAT_FLOAT div_float_float = new DIV_FLOAT_FLOAT();
		Heading h = relation1.getHeading();
		Set<NameIdentifier> partitioningKeys = h.subHeadingExcluding(floatingPointDependentAttributes.keySet(), true).getAttributeIdentifiersMapping().keySet();
		Map<Map<NameIdentifier, ValueBuffer>, Collection<TupleBuffer>> r1partitioned = DbmsRelationImplementation.partitionByIdentifier(relation1.getTupleCollectionBuffer().getTupleBufferCollection(), partitioningKeys);
		Map<Map<NameIdentifier, ValueBuffer>, Collection<TupleBuffer>> r2partitioned = DbmsRelationImplementation.partitionByIdentifier(relation2.getTupleCollectionBuffer().getTupleBufferCollection(), partitioningKeys);

		for (Entry<Map<NameIdentifier, ValueBuffer>, Collection<TupleBuffer>> r1pme : r1partitioned.entrySet()) {
			Collection<TupleBuffer> r1tuples = r1pme.getValue();
			Collection<TupleBuffer> r2tuples = r2partitioned.get(r1pme.getKey());
			if (r2tuples == null) {
				// difference is not exclusively due to floating point roundings because some combo of non-floating point values is missing
				return false;
			}
			if (r1tuples.size() > 1 || r2tuples.size() > 1) {
				// not readily decidable which tuples should be compared to which
				// return false for the time being
				return false;
			}
			TupleBuffer r1tuple = r1tuples.iterator().next();
			TupleBuffer r2tuple = r2tuples.iterator().next();
			// now iterate over the fp-dependent attributes and do comparisons for each such attribute between the two tuples
			// note that some attributes may be interval-typed with 2 floats, or nonscalar types with floats in their heading
			for (Entry<NameIdentifier, TypeDeclaration> fpame : floatingPointDependentAttributes.entrySet()) {
				final NameIdentifier attributeName = fpame.getKey();
				final TypeDeclaration attributeTypeDeclaration = fpame.getValue();
				ValueBuffer v1 = r1tuple.valueBuffer(attributeName);
				ValueBuffer v2 = r2tuple.valueBuffer(attributeName);
				if (attributeTypeDeclaration.isScalar()) {
					if (attributeTypeDeclaration.isIntervalTyped()) {
						if (!floatIntervalCloseEnough(div_float_float, v1, v2)) {
							return false;
						}
					} else {
						if (attributeTypeDeclaration.getTypeNameIdentifier().equalsNameIdentifier(TYPENAMES.AVERAGE_I)) {
							v1 = DbmsAverageImplementation.getInstance().getComponentValue(POSSREPCOMPONENTNAMES.VALUE, v1);
							v2 = DbmsAverageImplementation.getInstance().getComponentValue(POSSREPCOMPONENTNAMES.VALUE, v2);
						}
						if (!floatsCloseEnough(div_float_float, v1, v2)) {
							return false;
						}
					}
				} else {
					// theoretically need to recurse this within-margins comparisons to the RVA/TVA values concerned.
					// For the time being, we just skip that and return false
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * @param args
	 *            options that can be passed to the TestScenario. Possible options are {@link #HALTONFIRSTMISMATCH}, {@link DBConnectionProperties#HOST HOST}, {@link DBConnectionProperties#PORT PORT} and {@link #FILE}
	 */
	public static void main (String[] args) {

		final MyCommandLineParameters commandLineParameters = MyCommandLineParameters.getCommandLineParameters(args, FILE, DBConnectionProperties.getInstance());
		SiraPriseServer server = new SiraPriseServer(commandLineParameters.getProperty(DBConnectionProperties.HOST, DBConnectionProperties.HOSTDEFAULT), Integer.parseInt(commandLineParameters.getProperty(DBConnectionProperties.PORT, DBConnectionProperties.PORTDEFAULT)), Integer.parseInt(commandLineParameters.getProperty(DBConnectionProperties.MONITORPORT, DBConnectionProperties.MONITORPORTDEFAULT)));
		// get a DB connection
		DBConnection dbc = null;
		boolean retry;
		do {
			retry = false;
			try {
				dbc = new OneShotDBConnection(server, "", null); //$NON-NLS-1$

				try {
					new TestScenario(dbc, commandLineParameters.getProperty(FILE), Boolean.parseBoolean(commandLineParameters.getProperty(HALTONFIRSTMISMATCH, HALTONFIRSTMISMATCHDEFAULT))).run();
				} finally {
					dbc.close();
				}
			} catch (DBException e1) {
				if (e1.getErrorCode() == 454) { // max connections exceeded
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {

					}
					retry = true;
				} else {
					e1.printStackTrace(System.err);
				}
			} catch (IOException e) {
				e.printStackTrace(System.err);
			} catch (CommunicationProtocolException e) {
				e.printStackTrace(System.err);
			}
		} while (retry);
	}

	/**
	 * 
	 */
	private DBConnection dbc;

	/**
	 * 
	 */
	private boolean haltOnFirstResultMismatch;

	/**
	 * 
	 */
	private String scenarioName;

	/**
	 * Creates the TestScenario
	 * 
	 * @param dbc
	 *            the db connection
	 * @param scenarioName
	 *            the scenario name
	 * @param haltOnFirstResultMismatch
	 *            true if the test should stop on the first result mismatch
	 */
	public TestScenario (DBConnection dbc, String scenarioName, boolean haltOnFirstResultMismatch) {
		this.dbc = dbc;
		this.scenarioName = scenarioName;
		this.haltOnFirstResultMismatch = haltOnFirstResultMismatch;
	}

	/**
	 * @param heading
	 * @return
	 */
	private Map<NameIdentifier, TypeDeclaration> getFloatingPointDependentAttributes (Heading heading) {
		Map<NameIdentifier, TypeDeclaration> flpDependentAttributes = new HashMap<>();
		for (Entry<NameIdentifier, TypeDeclaration> hme : heading.getAttributeIdentifiersMapping().entrySet()) {
			if (hme.getValue().getTypeNameIdentifier().equalsNameIdentifier(TYPENAMES.FLOAT_I) //
					|| hme.getValue().getTypeNameIdentifier().equalsNameIdentifier(TYPENAMES.AVERAGE_I) //
					|| hme.getValue().isIntervalTyped() && hme.getValue().getTypeNameIdentifier().equalsNameIdentifier(NameIdentifier.get(TYPENAMES.FLOAT + TYPENAMES.INTERVALTYPENAMESUFFIX)) //
					|| (!hme.getValue().isScalar()) && getFloatingPointDependentAttributes(((NonScalarTypeDeclaration) hme.getValue()).getHeading()).size() > 0) {
				flpDependentAttributes.put(hme.getKey(), hme.getValue());
			}
		}
		return flpDependentAttributes;
	}

	/**
	 * @param results
	 * @param resultMisMatchDetected
	 * @param tx
	 * @param operationDetailElement
	 * @param label
	 * @return
	 * @throws ConnectionClosedException
	 */
	private boolean processInquiryTag (Collection<MyReadOnlyEntry<String, TestResultDetails>> results, DBTransaction tx, final Node operationDetailElement, String label) throws ConnectionClosedException {
		boolean resultMisMatchDetected = false;
		final NodeList inquiryDetailsElements = ((Element) operationDetailElement).getElementsByTagName("*"); //$NON-NLS-1$
		final String commandText = inquiryDetailsElements.item(0).getTextContent().trim(); // The Query element's content
		final Element expectedResultsElement = (Element) inquiryDetailsElements.item(1); // ExpectedResult
		final NodeList expectedResultsDetailsElements = expectedResultsElement.getElementsByTagName("*"); //$NON-NLS-1$
		String expectedRelationValue;
		long expectedExceptionType;
		// long expectedErrorNumber1;
		Collection<Long> expectedErrorNumbers = new LinkedList<>();
		String expectedExceptionClassName;
		String expectedExceptionMessage = null;
		// theoretically, can be empty, but there should be a warning about this
		if (expectedResultsDetailsElements.getLength() > 0) {
			if (expectedResultsDetailsElements.item(0).getNodeName().endsWith(":Relation")) { //$NON-NLS-1$
				expectedRelationValue = expectedResultsDetailsElements.item(0).getTextContent().trim();
				expectedExceptionType = 0;
				// expectedErrorNumber = 0;
				expectedExceptionClassName = null;
				// expectedExceptionMessage = null;
			} else {
				expectedRelationValue = null;
				NodeList expectedExceptionDetailItems = ((Element) expectedResultsDetailsElements.item(0)).getElementsByTagName("*"); //$NON-NLS-1$
				expectedExceptionType = Long.parseLong(expectedExceptionDetailItems.item(0).getTextContent().trim()); // ExceptionType tag
				int exceptionClassNameItemIndex;
				if (expectedExceptionDetailItems.getLength() > 1 && expectedExceptionDetailItems.item(1).getNodeName().endsWith(":ErrorNum")) { //$NON-NLS-1$
					final String allowableExpectedExceptionsCSV = expectedExceptionDetailItems.item(1).getTextContent().trim();
					expectedErrorNumbers = new CollectionTransformer<String, Long>(new StringToLongTransformer()).transformToSet(MyString.getNamesInEntryOrderWithoutDuplicates(allowableExpectedExceptionsCSV, ','));
					// expectedErrorNumber = Long.parseLong(expectedExceptionDetailItems.item(1).getTextContent().trim());
					exceptionClassNameItemIndex = 2;
				} else {
					// expectedErrorNumber = 0;
					exceptionClassNameItemIndex = 1;
				}
				int exceptionMessageItemIndex;
				if (expectedExceptionDetailItems.getLength() > exceptionClassNameItemIndex && expectedExceptionDetailItems.item(exceptionClassNameItemIndex).getNodeName().endsWith(":ExceptionClass")) { //$NON-NLS-1$
					expectedExceptionClassName = expectedExceptionDetailItems.item(exceptionClassNameItemIndex).getTextContent();
					exceptionMessageItemIndex = exceptionClassNameItemIndex + 1;
				} else {
					expectedExceptionClassName = null;
					exceptionMessageItemIndex = exceptionClassNameItemIndex;
				}
				if (expectedExceptionDetailItems.getLength() > exceptionMessageItemIndex && expectedExceptionDetailItems.item(exceptionMessageItemIndex).getNodeName().endsWith(":ExceptionMessage")) { //$NON-NLS-1$
					expectedExceptionMessage = expectedExceptionDetailItems.item(exceptionMessageItemIndex).getTextContent();
					// } else {
					// expectedExceptionMessage = null;
				}
			}

		} else {
			System.out.println(MyMessageFormat.format("\r\nWarning : empty expected results found for query {0}", new String[] { commandText })); //$NON-NLS-1$
			expectedRelationValue = null;
			expectedExceptionType = 0;
			// expectedErrorNumber = 0;
			expectedExceptionClassName = null;
			// expectedExceptionMessage = null;
		}
		System.out.print('.');
		Relation result;
		try {
			result = tx.execQuery(commandText);
			if (expectedRelationValue != null) {
				try {
					Relation[] differences = tx.execQueries(new String[] { "MINUS(" + result.printValueEscapedWithTypeNames() + ',' + expectedRelationValue + ")", "MINUS(" + expectedRelationValue + ',' + result.printValueEscapedWithTypeNames() + ")" }); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

					if (differences[0].size() > 0 || differences[1].size() > 0) {
						final Map<NameIdentifier, TypeDeclaration> floatingPointDependentAttributes = getFloatingPointDependentAttributes(differences[0].getHeading());
						if (floatingPointDependentAttributes.size() > 0) {
							if (resultsWithinExpectedMarginForFloats(differences[0], differences[1], floatingPointDependentAttributes)) {
								// FP Results sufficiently close to consider them equal
							} else {
								resultMisMatchDetected = true;
								results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new DifferingResultDetails(expectedRelationValue, result, differences[0], differences[1])));
							}
						} else {
							resultMisMatchDetected = true;
							results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new DifferingResultDetails(expectedRelationValue, result, differences[0], differences[1])));
						}
					}
				} catch (ErrorMessageException e) {
					resultMisMatchDetected = true;
					results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ComparisonOfResultsFailed(e, result, expectedRelationValue)));
				}
			} else {
				resultMisMatchDetected = true;
				results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new UnexpectedRelation(expectedResultsDetailsElements.item(0).toString(), result)));
			}
		} catch (ErrorMessageException e) {
			// check the returned exception against the expected exception
			final ServerErrorMessage serverErrorMessage = e.getServerErrorMessage();
			if (expectedRelationValue == null) {
				final int obtainedExceptionType = serverErrorMessage.getMessageTypeID();
				if (obtainedExceptionType != expectedExceptionType) {
					resultMisMatchDetected = true;
					results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionTypeMismatch(expectedExceptionType, obtainedExceptionType, serverErrorMessage.getErrorClass())));
				} else {
					if (expectedExceptionType != NonSiraPriseErrorMessageType.MESSAGETYPEID && (!expectedErrorNumbers.contains(new Long(serverErrorMessage.getErrorNumber())))) {
						resultMisMatchDetected = true;
						results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionNumberMismatch(expectedErrorNumbers, serverErrorMessage.getErrorNumber(), expectedExceptionMessage, serverErrorMessage.getClientResponseText())));
					} else {
						if (expectedExceptionType == NonSiraPriseErrorMessageType.MESSAGETYPEID && !(((NonSiraPriseErrorMessage) serverErrorMessage).getJavaThrowableClassName().equals(expectedExceptionClassName))) {
							resultMisMatchDetected = true;
							results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new ExceptionClassMismatch(expectedExceptionClassName, ((NonSiraPriseErrorMessage) serverErrorMessage).getJavaThrowableClassName())));
						} else {

						}
					}
				}
			} else {
				resultMisMatchDetected = true;
				results.add(new MyReadOnlyEntry<String, TestResultDetails>(getOperationIdentifier(label, commandText), new UnexpectedExceptionDetails(expectedRelationValue, e)));
			}
		}
		return resultMisMatchDetected;
	}

	/**
	 * @param results
	 * @param resultMisMatchDetected
	 * @param tx
	 * @param transactionDetailElements
	 * @return
	 * @throws ConnectionClosedException
	 */
	private boolean processTransactionTag (Collection<MyReadOnlyEntry<String, TestResultDetails>> results, DBTransaction tx, final NodeList transactionDetailElements) throws ConnectionClosedException {
		boolean resultMisMatchDetected = false;
		for (int j = 0; j < transactionDetailElements.getLength() && !(haltOnFirstResultMismatch && resultMisMatchDetected); j++) {
			final Node transactionDetailElement = transactionDetailElements.item(j); // The <Operation> element
			if (((Element) transactionDetailElement).getNodeName().toUpperCase().endsWith(TAGNAME_OPERATION)) {
				String label = ((Element) transactionDetailElement).hasAttribute(ATTRIBUTENAME_LABEL) ? ((Element) transactionDetailElement).getAttribute(ATTRIBUTENAME_LABEL) : ""; //$NON-NLS-1$
				// System.out.println(possibleOperationElement.getNextSibling().getNodeName());
				final NodeList operationDetailElements = ((Element) transactionDetailElement).getElementsByTagName("*"); //$NON-NLS-1$
				final Node operationDetailElement = operationDetailElements.item(0); // The <Assignment> or <Inquiry> element
				final String operationDetailElementName = operationDetailElement.getNodeName();
				switch (operationDetailElementName.substring(operationDetailElementName.indexOf(':') + 1).toUpperCase()) {
					case TAGNAME_COMMIT: {
						processCommitTag(tx, operationDetailElement);
						break;
					}
					case TAGNAME_ASSIGNMENT: {
						resultMisMatchDetected = processAssignmentTag(results, tx, operationDetailElement, label);
						break;
					}
					case TAGNAME_INQUIRY: {
						resultMisMatchDetected = processInquiryTag(results, tx, operationDetailElement, label);
						break;
					}
					default: {
						throw new IllegalArgumentException(operationDetailElementName);
					}
				}
			} else {
				if (((Element) transactionDetailElement).getNodeName().toUpperCase().endsWith(TAGNAME_NESTEDTRANSACTION)) {

					final Element possibleTransactionElement = (Element) transactionDetailElement;
					boolean autoCommit = possibleTransactionElement.hasAttribute(ATTRIBUTENAME_AUTOCOMMIT) ? Boolean.parseBoolean(possibleTransactionElement.getAttribute(ATTRIBUTENAME_AUTOCOMMIT)) : false;
					boolean transactionMode = possibleTransactionElement.hasAttribute(ATTRIBUTENAME_READONLY) ? Boolean.parseBoolean(possibleTransactionElement.getAttribute(ATTRIBUTENAME_READONLY)) : false;
					String transactionName = possibleTransactionElement.hasAttribute(ATTRIBUTENAME_NAME) ? possibleTransactionElement.getAttribute(ATTRIBUTENAME_NAME) : ""; //$NON-NLS-1$
					try {
						DBTransaction nestedtx = tx.startNestedTransaction(autoCommit, transactionMode);
						resultMisMatchDetected = processTransactionTag(results, nestedtx, possibleTransactionElement.getElementsByTagName("*")); //$NON-NLS-1$
					} catch (ErrorMessageException e) {
						results.add(new MyReadOnlyEntry<String, TestResultDetails>(transactionName, new UnexpectedExceptionDetails(e)));
					}

				}
			}
		}

		// end the transaction
		tx.end(true);

		return resultMisMatchDetected;
	}

	/**
	 * @return
	 * @throws ConnectionClosedException
	 */
	Collection<MyReadOnlyEntry<String, TestResultDetails>> runTest ( ) throws ConnectionClosedException {
		// get the factory
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

		Collection<MyReadOnlyEntry<String, TestResultDetails>> results = new LinkedList<>();
		try {
			// Using factory get an instance of document builder
			// parse using builder to get DOM representation of the XML file
			// get the root element
			// get a nodelist of elements
			NodeList elementList = dbf.newDocumentBuilder().parse(MyResource.getResourceInputStream(getClass(), scenarioName + ".xml")).getDocumentElement().getElementsByTagName("*"); //$NON-NLS-1$ //$NON-NLS-2$
			boolean resultMisMatchDetected = false;
			// Have to use * because we want Operation elements, regardless of namespace prefix.
			// "Operation" gets us back nothing because it differs from myprefix:Operation.
			// Cannot specify "myprefix:Opeation" either, because then anyone using hisprefix:Operation would see his scenario not processed (hisprefix differs from myprefix"
			// So we have to take all elements
			// And then we get the next "pleasant" surprise : All elements are returned regardless of nesting level, i.e. the hierarchical structure of the XML document is completely ignored by this method.
			for (int i = 0; i < elementList.getLength() && !(haltOnFirstResultMismatch && resultMisMatchDetected); i++) {
				// get the possible <Transaction> element
				final Element possibleTransactionElement = (Element) elementList.item(i);
				if (possibleTransactionElement.getNodeName().toUpperCase().endsWith(TAGNAME_TRANSACTION)) {
					//						String transactionName = possibleTransactionElement.getAttribute("name"); //$NON-NLS-1$
					boolean autoCommit = possibleTransactionElement.hasAttribute(ATTRIBUTENAME_AUTOCOMMIT) ? Boolean.parseBoolean(possibleTransactionElement.getAttribute(ATTRIBUTENAME_AUTOCOMMIT)) : false;
					boolean transactionMode = possibleTransactionElement.hasAttribute(ATTRIBUTENAME_READONLY) ? Boolean.parseBoolean(possibleTransactionElement.getAttribute(ATTRIBUTENAME_READONLY)) : false;
					DBTransaction tx = dbc.startTransaction(autoCommit, TransactionMode.getTransactionMode(transactionMode));
					final NodeList transactionDetailElements = possibleTransactionElement.getElementsByTagName("*"); //$NON-NLS-1$
					resultMisMatchDetected = processTransactionTag(results, tx, transactionDetailElements);
				}
			}
		} catch (ParserConfigurationException | SAXException | IOException | NotFoundException | ErrorMessageException e) {
			e.printStackTrace(System.err);
		}

		System.out.println();
		return results;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run ( ) {
		try {
			Collection<MyReadOnlyEntry<String, TestResultDetails>> results = runTest();
			for (MyReadOnlyEntry<String, TestResultDetails> me : results) {
				System.out.println();
				System.out.println(me.getKey());
				me.getValue().log(System.out);
			}
			System.out.println();
			System.out.println(MyMessageFormat.format("{0} problems found.", new String[] { Integer.toString(results.size()) })); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			e.printStackTrace(System.err);
		}
	}
}
